/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2013  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.88e                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_R32C.c for RENESAS R32C with IAR compiler
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "ior32c121.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* Main clock frequency         */
  #define OS_FSYS 16000000uL        /* External oscillator          */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER OS_FSYS     /* We use external clock        */
#endif

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_FSYS      /* We use external clock        */
#endif

#ifndef   INIT_PLL
  #define INIT_PLL (1)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/
#ifndef   OS_UART
  #define OS_UART 0
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400uL
#endif

/****** End of configurable options *********************************/

/*********************************************************************
*
*       Check configuration and select sfr include file
*
**********************************************************************
*/

#define _OS_UART_USED (OS_VIEW_ENABLE && (OS_UART == 0) || (OS_UART == 1))

#if _OS_UART_USED
  // Assigne SFRs and interrupt vectors for UART
  #if (OS_UART == 0)
    #define _UARTX_RX      (UART0_RX)
    #define _UARTX_TX      (UART0_TX)
    #define _UxTB          (U0TB)
    #define _UxRB          (U0RB)
    #define _UxC1          (U0C1)
    #define _UxMR          (U0MR)
    #define _UxC0          (U0C0)
    #define _UxBRG         (U0BRG)
    #define _UxC1          (U0C1)
    #define _SxRIC         (S0RIC)
    #define _SxTIC         (S0TIC)
    #define _TX_PORT_BIT   (3)
    #define _UART_PORT_PS  (P6_3S)
  #elif (OS_UART == 1)
    #define _UARTX_RX      (UART1_RX)
    #define _UARTX_TX      (UART1_TX)
    #define _UxTB          (U1TB)
    #define _UxRB          (U1RB)
    #define _UxC1          (U1C1)
    #define _UxMR          (U1MR)
    #define _UxC0          (U1C0)
    #define _UxBRG         (U1BRG)
    #define _UxC1          (U1C1)
    #define _SxRIC         (S1RIC)
    #define _SxTIC         (S1TIC)
    #define _TX_PORT_BIT   (7)
    #define _UART_PORT_PS  (P6_7S)
  #endif
#endif  // _OS_UART_USED

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  TA0   = (OS_PCLK_TIMER/1000 - 1);  // Generate Interrupts every 1 ms
  TA0MR = 0x80;            // Timer mode register
                           // bit 01 : 00 Timer mode
                           // bit 2  :  0 No puls output
                           // bit 34 : 0X No gate function
                           // bit 5  : 0
                           // bit 67 : 01 clock source : f/2n
  TA0IC  = 1;              // Timer interrupt register, priority 1 (lowest)
  TABSR |= 1;              // Set Timer count start flag

  TA0IC_bit.ILVL = 1;      // Enable timer interrupts, lowest priority
  OS_COM_Init();           // Initialize UART for embOS Viewer
}

/*********************************************************************
*
* OS_Idle()
*
* Please note:
* This is basically the "core" of the idle loop.
* This core loop can be changed, but:
* The idle loop does not have a stack of its own, therefore no
* functionality should be implemented that relies on the stack
* to be preserved. However, a simple program loop can be programmed
* (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to exec
  for (;;) {            // Nothing to do ... wait for interrupt
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  unsigned int T_cnt;
  OS_U32 Time;

  T_cnt = TA0;
  Time = OS_Time + 1;
  if (TA0IC & (1 << 3)) {   // If timer interrupt pending ...
    T_cnt = TA0;            // adjust result
    Time++;
  }
  return (OS_PCLK_TIMER/1000) * Time - T_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick()
*       The OS system timer interrupt
*/
#pragma vector = (TIMER_A0)
static __interrupt void OS_ISR_Tick (void) {
  OS_EnterNestableInterrupt();
  OS_TICK_Handle();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView
*
**********************************************************************
*/
#if _OS_UART_USED
#define BAUDDIVIDE (((OS_PCLK_UART + (OS_BAUDRATE*8L)) / (OS_BAUDRATE*16L)) - 1)

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  _UxTB = c;
}

/*********************************************************************
*
*       OS_ISR_rx()
*       OS UART rx interrupt handler
*/
#pragma vector = (_UARTX_RX)
static __interrupt void OS_ISR_rx (void) {
  int Data;
  OS_EnterNestableInterrupt();  /*  We will enable interrupts */
  Data = _UxRB;
  if (Data & 0x6000) {   /* Check if errors occurred */
    _UxC1 &= ~(1 << 2);   /* disable Rx               */
    _UxC1 |=  (1 << 2);   /* enable Rx                */
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_tx()
*       OS UART tx interrupt handler
*/
#pragma vector= (_UARTX_TX)
static __interrupt void OS_ISR_tx (void) {
  OS_EnterNestableInterrupt();
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {
  _UxMR   = 0x00;               //  Lock Sio, error reset
  _UxC0   = 0x02                //  Use f2n as clock source
          | (0 << 2)            //  Reserved, always write 0
          | (0 << 3)            //  TxEPT, RO, write 0
          | (1 << 4)            //  CRD:   1 => Disable CTS
          | (0 << 5)            //  NCH:   1 => TxD Output is push-pull
          | (0 << 6)            //  CKPOL: don't care for UART mode, write 0
          | (0 << 7)            //  UFORM: 0 => LSB first
          ;
  _UxBRG  = BAUDDIVIDE;         //  Calculated Baudrate
  _UxC1   = 0;                  //  Lock Rx and Tx
  _UxMR   = 0x05                //  8  Data
          +(0 << 4)             //  0: 1 Stop-Bit (1= 2 Stop-Bits)
          +(0 << 5)             //  X: Parity select
          +(0 << 6)             //  1: parity enable
          +(0 << 7);            //  0: no sleep
  _UxC1   = 5;                  //  enable reception and transmission
  _SxRIC  = 1;                  //  Enable UART Rx interrupts
  _SxTIC  = 1;                  //  Enable UART Tx interrupts
  PD6   |= (1 << _TX_PORT_BIT); //  Direction register   (0=IN 1=OUT)
  P6    |= (1 << _TX_PORT_BIT); //  Register  (0=Low, 1= High)
  _UART_PORT_PS = 3;            //  Enable TxD function
}

#else  /* No communication routines */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);           /* Not used, avoid warning */
    OS_COM_ClearTxActive();  /* Let OS know that transmitter is not busy  */
  }
#endif // _OS_UART_USED

/*********************************************************************
*
*       __low_level_init()
*
*       Initialize clock generation and pll
*
*       Has to be modified, if another CPU clock frequency should be
*       used. This function is called during startup and
*       has to return 1 to perform segment initialization.
*
*       Because variables are not initialized before __low_level_init() is called,
*       no access or call which relies on any variable can be used here.
*/
int __low_level_init(void);  // Avoid "no ptototype" warning
int __low_level_init(void) {

#if INIT_PLL
  unsigned int n;

  // Initialise CPU
  // - single chip mode
  // - no external bus
  // - external clock    16 MHz
  // - PLL clock        128 MHz
  // - CPU clock         64 MHz
  // - Base clock        64 MHz
  // - Peripheral clock  32 MHz
  prcr  = 0xFF;     // enable write to control registers
  prcr2 = 0x80;     // enable write to CM3 register
  prr   = 0xAA;     // enable write to CCR, FMCR, PBC register

  pm2   = 0x02;     // processor mode register 2: enable clock change
  cm0   = 0x08;     // system clock control register 0
  cm1   = 0x20;     // system clock control register 1
  cm2   = 0x00;     // oscillation stop detect register
  cm3   = 0x00;     // low speed mode clock control register
  tcspr = 0x00;     // count source prescaler register
  cpsrf = 0x00;     // clock prescaler reset register
  pbc   = 0x0504;   // peripheral bus control register
                    //  value depends on setting of CCR register
  ccr   = 0x1F;     // clock control register
                    // 11: Base clock divide ratio:       /2
                    // 11: CPU clock divide ratio:        /1
                    // 01: Peripheral clock divide ratio: /2
  prcr  = 0xFF;
  plc0  = 0x45;     // pll control register 0: 01000101
  prcr  = 0xFF;
  plc1  = 0x03;     // pll control register 1
  seo   = 0;        // pll mode
  for (n = 0; n < 0x8000; n++) {
    asm("NOP");
  }
  prr   = 0xAA;     // enable write to CCR, FMCR, PBC register
  bcs   = 0;        // base clock source is PLL

  prcr  = 0xFF;
  pm3   = 0x40;     // peripheral clock = PLL/4
  pm2   = 0x40;     // processor mode register 2: disable clock change
                    // count source for f2n is main clock
  prr   = 0x00;     // disable write to CCR, FMCR, PBC register
#endif // INIT_PLL
  //
  // Ensure external clock (f2n) is enabled as source for timer (and UART)
  //
  prcr  = (0 << 0)    // PRC0: 1 => Enable write to CM0, CM1, CM2, PM3
        | (1 << 1)    // PRC1: 1 => Enable write to PM0, PM2, INVC0, INVC1 register
        | (0 << 2)    // PRC2: 1 => Enable write to PD9, P9_iS, PLC0, PLC1
        ;
  PM2  |= (1 << 6);   // PM26: 1 => select Xin as f2n clock source
  tcspr = 0           // Divider for f2n: 0 => do not divide, divide by 1
        | (1 << 7)    // CST: 1 => Enable f2n clock
        ;
  prcr  = 0x00;       // Disable write to control registers
  //
  // Other initialization may be added here if required
  //
  return 1;           // Always return 1 to enable segment initilization !
}

/****** End Of File *************************************************/

